-- local E = 40 -- GJ
-- local P = 600 -- kW
local mod_name = "__RITEG__"
local ln_2 = 0.69314718
local rtg_names_list = {'RITEG-1'}
local RTG_prop = 
  {
    ['RITEG-1'] = {energy = 40*10^9},
    ['RITEG-2'] = {energy = 40*10^9 * 4}
  }

script.on_configuration_changed(function(data)
  try_init ()
  update_recipes ()
  add_all_rtgs_to_table ()
  update_settings ()
end)

-- update riteg settings
function update_settings (event)
  global.rtgs_settings["enabled"]   = settings.global[ mod_name .. "autodeconstruct-is-enabled" ].value
  global.rtgs_settings["threshold"] = settings.global[ mod_name .. "autodeconstruct-health-threshold" ].value

  global.rtgs_settings["max_healths"] = {}
  for i, e in pairs (rtg_names_list) do
    global.rtgs_settings["max_healths"][e] = game.entity_prototypes[e].max_health
  end
end

-- call update_settings when on_runtime_mod_setting_changed
script.on_event(defines.events.on_runtime_mod_setting_changed, update_settings)


script.on_init(function(data)
  try_init ()
  update_recipes ()
  update_settings ()
end)

function is_element_in_list (element, list)
  for i, e in pairs (list) do
    if e == element then return true end
  end
  return false
end

function update_recipes ()
  for i, force in pairs (game.forces) do
    force.reset_technology_effects()
  end
end

function add_all_rtgs_to_table ()
local tick = game.tick
  for i, surface in pairs (game.surfaces) do
    for j, rtg_name in pairs (rtg_names_list) do
      local found_entities = surface.find_entities_filtered{name= 'rtg_name'}
      for k, entity in pairs (found_entities) do
        
        local rtg = make_rtg_prototype {name = rtg_name, tick = tick, entity = entity}
        
        -- {name = rtg_name, 
          -- first_tick = tick, 
          -- -- first_energy = 40*10^9, -- 40 GJ
          -- first_energy = RTG_prop[rtg_name].energy, -- 40 GJ
          -- -- first_power = 600*10^3, -- 600 kW
          -- first_power = entity.health*10^3, -- 600 kW
          -- entity = entity}
        -- rtg['half_life'] = E * ln_2 / P
        -- rtg['half_life'] = rtg['first_energy'] * ln_2 / rtg['first_power']
        -- rtg['power'] = power_of_RITEG(rtg)
        table.insert(global.rtgs, rtg)
        global.total_rtgs = global.total_rtgs + 1
      end
    end
  end
  log('global.total_rtgs = ' .. global.total_rtgs)
end

function make_rtg_prototype (data)
  local rtg = {name = data.name, 
  first_tick = data.tick, 
  -- first_energy = 40*10^9, -- 40 GJ
  first_energy = RTG_prop[data.name].energy, -- 40 GJ
  -- first_power = 600*10^3, -- 600 kW
  entity = data.entity,
  first_power = data.entity.health*10^3 -- 600 kW
  }
  rtg['half_life'] = rtg['first_energy'] * ln_2 / rtg['first_power']
  rtg['power'] = power_of_RITEG(rtg)
  
  log ('starting power: ' .. rtg['power']*60 .. ' half life: '..rtg['half_life'] .. ' starting energy: '..rtg['first_energy'])
  return rtg
end

function try_init ()
  if not global then global = {} end
  if not global.rtgs then global.rtgs = {} end
  if not global.tick_id then global.tick_id = 1 end
  if not global.total_rtgs then global.total_rtgs = 0 end
  if not global.rtgs_settings then global.rtgs_settings = {} end
end

script.on_event(defines.events.on_built_entity, function(event)
  for j, rtg_name in pairs (rtg_names_list) do
    if	(event.created_entity.name == rtg_name) then 
      onBuildHandler(event) 
    end
  end
end)

script.on_event(defines.events.on_robot_built_entity, function(event)
  for j, rtg_name in pairs (rtg_names_list) do
    if	(event.created_entity.name == rtg_name) then 
      onBuildHandler(event) 
    end
  end
end)

function onBuildHandler(event) 
  local tick = game.tick  
  local entity = event.created_entity
  -- local health = entity.health
  local rtg_name = entity.name
  local rtg = make_rtg_prototype {name = rtg_name, tick = tick, entity = entity}
  
  -- {name = rtg_name, 
    -- first_tick = tick, 
    -- -- first_energy = 40*10^9, -- 40 GJ
    -- first_energy = RTG_prop[rtg_name].energy, -- 40 GJ
    -- -- first_power = 600*10^3, -- 600 kW
    -- first_power = entity.health*10^3, -- 600 kW
    -- entity = entity}
  -- rtg['half_life'] = rtg['first_energy'] * ln_2 / rtg['first_power']
  -- log ('half_life = ' .. rtg['half_life'])
  -- rtg['power'] = power_of_RITEG(rtg)
  table.insert(global.rtgs, rtg)
  global.total_rtgs = global.total_rtgs + 1
end

script.on_event(defines.events.on_tick, function(event)
  if global.total_rtgs == 0 then return end
  if global.tick_id > global.total_rtgs then
    global.tick_id = 1
  end
  local rtg = global.rtgs[global.tick_id]
  if not (rtg and rtg.entity.valid)  then
    -- remove this element from tha table and return
    -- printAll('remove rtg')
    table.remove(global.rtgs, global.tick_id)
    global.total_rtgs = global.total_rtgs - 1
    return
  end

  local tick = game.tick
  local power = power_of_RITEG(rtg)
  rtg.entity.power_production = power
  rtg.power = power
  
  -- new code
  -- printAll('health: '.. rtg.entity.health .. ' power: '.. power)
  if rtg.entity.health * (50/3) > power then -- 50/3 is 10000 Joule/tick (it's 600 000 Watt or 600 kW) divide thrue 600 Watt
    -- printAll('health was: '.. rtg.entity.health .. ' power: '.. power /(50/3) .. ' power: '.. math.floor(power /(50/3)))
    rtg.entity.health =  math.floor(power/(50/3))
  end
  -- end of new code

  -- autodeconstuct
  if global.rtgs_settings["enabled"] == true then
    local threshold_health = ( global.rtgs_settings["threshold"] * global.rtgs_settings["max_healths"][rtg.entity.name]) / 100
    if rtg.entity.health < threshold_health then
      rtg.entity.order_deconstruction(rtg.entity.force)
    end
  end
  -- end of autodeconsturt
  
  --printAll('RITEG [' .. global.tick_id .. '] has power ' .. rtg['power'] .. 'W and half_life = ' .. rtg['half_life'] .. 's')
  global.tick_id = global.tick_id + 1
end)

function power_of_RITEG(rtg)
  return rtg['first_power'] / (2^((game.tick-rtg['first_tick'])/(rtg['half_life'] * 60)))/60
end

function remaining_energy (rtg)
  -- E = P_0 * t_HL / (ln(2))

end

function half_life (rtg)
  return rtg['first_energy'] * ln_2 / rtg['first_power']
end

function printAll(text)
  for i in pairs (game.players) do
    game.players[i].print(text)
  end
end